// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawnlibs.hpp"

namespace Dawn {
  template<typename ...A>
  class Event {
    private:
      int32_t nextId = 0;
      std::unordered_map<int32, std::function<void(A...)>> listeners;

    public:
      Event() {

      }

      void emit(A ...args) {
        auto copy = listeners;
        for(auto &pair : copy) {
          pair.second(args...);
        }
      }

      std::function<void()> listen(const std::function<void(A...)> listener) {
        int32_t id = nextId++;
        listeners[id] = listener;
        return [this, id]() {
          listeners.erase(id);
        };
      }

      virtual ~Event() {
        listeners.clear();
      }
  }
}